""" x-invoice

Parsing X-Invoice files
"""

__version__ = "0.0.1"


import pathlib

import click
import pyperclip
from bs4 import BeautifulSoup

RECIPIENT_MAP = {
    "calvino": "Céline",
    "pappas": "Babis",
    "slesarenko": "Slava",
    "makro": "Makro",
}


def open_invoice(file_path):
    with open(file_path, "r") as handle:
        return BeautifulSoup(handle, "xml")


def _drill_down(soup, tags):
    current_tag, *rest_tags = tags
    findings = soup.find_all(current_tag)
    if not findings:
        yield None
    elif not rest_tags:
        yield from (f.string for f in findings)
    else:
        for child in findings:
            yield from drill_down(child, rest_tags)


def drill_down(soup, *tag_lists):
    for tags in tag_lists:
        result = list(_drill_down(soup, tags))
        if result != [None]:
            break
    return result


def get_recipient(soup):
    results = drill_down(
        soup,
        ["cac:DeliveryParty", "cac:PartyName", "cbc:Name"],
        [
            "ram:ApplicableHeaderTradeDelivery",
            "ram:ShipToTradeParty",
            "ram:Name",
        ],
    )
    return results[0]


def get_recipient_short_name(soup):
    full_text = get_recipient(soup)
    if full_text is None:
        return "+++ UNKNOWN +++"
    full_text = full_text.lower()
    for key, value in RECIPIENT_MAP.items():
        if key in full_text:
            return value
    return "CPI"


def get_items(soup):
    results = drill_down(
        soup,
        ["cac:InvoiceLine", "cac:Item", "cbc:Name"],
        [
            "ram:IncludedSupplyChainTradeLineItem",
            "ram:SpecifiedTradeProduct",
            "ram:Name",
        ],
    )
    return results


def parse(file_path):
    soup = open_invoice(file_path)
    recipient = get_recipient_short_name(soup)
    items = get_items(soup)
    lines = [f"for {recipient}:"] + items
    text = "\n".join((str(line) for line in lines))
    pyperclip.copy(text)
    return text


@click.command()
@click.argument("file_path", type=click.Path(exists=True))
def main(file_path):
    print(parse(file_path))
    pathlib.Path(file_path).unlink(missing_ok=True)
